@@ -30,19 +30,27 @@
 #define LLCP_DEFAULT_MIU 128
 
 struct near_p2p_driver {
-	const char *name;
-	const char *service_name;
+	char *name;
+	char *service_name;
 	const char *fallback_service_name;
-	near_bool_t single_connection;
+	bool single_connection;
 	int sock_type;
-	near_bool_t (*read)(int client_fd,
-				uint32_t adapter_idx, uint32_t target_idx,
-				near_device_io_cb cb);
+	gpointer user_data;
+
+	bool (*read)(int client_fd, uint32_t adapter_idx, uint32_t target_idx,
+						near_device_io_cb cb,
+						gpointer data);
 	int (*push)(int client_fd, uint32_t adapter_idx, uint32_t target_idx,
-			struct near_ndef_message *ndef, near_device_io_cb cb);
-	void (*close)(int client_fd, int err);
+						struct near_ndef_message *ndef,
+						near_device_io_cb cb,
+						gpointer data);
+	void (*close)(int client_fd, int err, gpointer data);
+	bool (*new_client)(char *service_name, int client_fd, gpointer data);
 };
 
+int phdc_init(void);
+void phdc_exit(void);
+
 int npp_init(void);
 void npp_exit(void);
 
