@@ -25,6 +25,7 @@
 
 #include <stdint.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <string.h>
 #include <sys/socket.h>
 
@@ -48,7 +49,7 @@
  *
  * TODO: check the nfc_data (simple NDEF or complete frame ?)
  *  */
-static near_bool_t snep_default_server_req_put(int client_fd, void *data)
+static bool snep_default_server_req_put(int client_fd, void *data)
 {
 	struct p2p_snep_data *snep_data = data;
 	struct near_device *device;
@@ -64,19 +65,22 @@ static near_bool_t snep_default_server_req_put(int client_fd, void *data)
 			snep_data->target_idx,
 			snep_data->nfc_data,
 			snep_data->nfc_data_length) < 0)
-		return FALSE;
+		return false;
 
 	device = near_device_get_device(snep_data->adapter_idx,
 			snep_data->target_idx);
-	if (device == NULL)
-		return FALSE;
+	if (!device)
+		return false;
 
 	records = near_ndef_parse_msg(snep_data->nfc_data,
 				snep_data->nfc_data_length, NULL);
 
 	near_device_add_records(device, records, snep_data->cb, 0);
 
-	return TRUE;
+	snep_data->nfc_data_length = 0;
+	snep_data->nfc_data = NULL;
+
+	return true;
 }
 
 /*
@@ -86,7 +90,7 @@ static near_bool_t snep_default_server_req_put(int client_fd, void *data)
  * !!! We check if the incoming NDEF looks like a handover frame,
  * because of Android 4.1.1 ...
  *  */
-static near_bool_t snep_default_server_req_get(int client_fd, void *data)
+static bool snep_default_server_req_get(int client_fd, void *data)
 {
 	struct p2p_snep_data *snep_data = data;
 
@@ -107,19 +111,21 @@ static near_bool_t snep_default_server_req_get(int client_fd, void *data)
 						NEAR_SNEP_ACC_LENGTH_SIZE);
 	}
 
-	return TRUE;
+	return true;
 }
 
 /* This function is a wrapper to push post processing read functions */
-static near_bool_t snep_default_read(int client_fd, uint32_t adapter_idx,
+static bool snep_default_read(int client_fd, uint32_t adapter_idx,
 							uint32_t target_idx,
-							near_tag_io_cb cb)
+							near_tag_io_cb cb,
+							gpointer data)
 {
 	DBG("");
 
 	return near_snep_core_read(client_fd, adapter_idx, target_idx, cb,
 						snep_default_server_req_get,
-						snep_default_server_req_put);
+						snep_default_server_req_put,
+						data);
 
 }
 
