@@ -61,10 +61,10 @@ struct near_adapter {
 	uint32_t poll_mode;
 	enum near_adapter_rf_mode rf_mode;
 
-	near_bool_t powered;
-	near_bool_t polling;
-	near_bool_t constant_poll;
-	near_bool_t dep_up;
+	bool powered;
+	bool polling;
+	bool constant_poll;
+	bool dep_up;
 
 	GHashTable *tags;
 	struct near_tag *tag_link;
@@ -79,6 +79,7 @@ struct near_adapter {
 	GList *ioreq_list;
 
 	guint presence_timeout;
+	guint dep_timer;
 };
 
 struct near_adapter_ioreq {
@@ -101,6 +102,9 @@ static void free_adapter(gpointer data)
 	if (adapter->presence_timeout > 0)
 		g_source_remove(adapter->presence_timeout);
 
+	if (adapter->dep_timer > 0)
+		g_source_remove(adapter->dep_timer);
+
 	g_free(adapter->name);
 	g_free(adapter->path);
 	g_hash_table_destroy(adapter->tags);
@@ -138,22 +142,14 @@ static char *rf_mode_to_string(struct near_adapter *adapter)
 
 static void polling_changed(struct near_adapter *adapter)
 {
-
-	near_dbus_property_changed_basic(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Polling",
-					DBUS_TYPE_BOOLEAN, &adapter->polling);
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Polling");
 }
 
 static void rf_mode_changed(struct near_adapter *adapter)
 {
-	const char *rf_mode = rf_mode_to_string(adapter);
-
-	if (rf_mode == NULL)
-		return;
-
-	near_dbus_property_changed_basic(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Mode",
-					DBUS_TYPE_STRING, &rf_mode);
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Mode");
 }
 
 static int adapter_start_poll(struct near_adapter *adapter)
@@ -165,14 +161,12 @@ static int adapter_start_poll(struct near_adapter *adapter)
 		DBG("Clearing tags");
 
 		g_hash_table_remove_all(adapter->tags);
-		__near_adapter_tags_changed(adapter->idx);
 	}
 
 	if (g_hash_table_size(adapter->devices) > 0) {
 		DBG("Clearing devices");
 
 		g_hash_table_remove_all(adapter->devices);
-		__near_adapter_devices_changed(adapter->idx);
 	}
 
 	DBG("Poll mode 0x%x", adapter->poll_mode);
@@ -189,35 +183,109 @@ static int adapter_start_poll(struct near_adapter *adapter)
 	if (err < 0)
 		return err;
 
-	adapter->polling = TRUE;
+	adapter->polling = true;
 
 	polling_changed(adapter);
 
 	return 0;
 }
 
-static void append_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_adapter *adapter = value;
-	DBusMessageIter *iter = user_data;
-
-	DBG("%s", adapter->path);
-
-	if (adapter->path == NULL)
-		return;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&adapter->path);
-}
-
-void __near_adapter_list(DBusMessageIter *iter, void *user_data)
-{
-	g_hash_table_foreach(adapter_hash, append_path, iter);
-}
-
-static void append_protocols(DBusMessageIter *iter, void *user_data)
+static gboolean property_get_mode(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
 {
 	struct near_adapter *adapter = user_data;
+	const char *rf_mode;
+
+	rf_mode = rf_mode_to_string(adapter);
+	if (!rf_mode)
+		return FALSE;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &rf_mode);
+
+	return TRUE;
+}
+
+static gboolean property_get_polling(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_adapter *adapter = user_data;
+	dbus_bool_t val;
+
+	val = adapter->polling;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_BOOLEAN, &val);
+
+	return TRUE;
+}
+
+static gboolean property_get_powered(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_adapter *adapter = user_data;
+	dbus_bool_t val;
+
+	val = adapter->powered;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_BOOLEAN, &val);
+
+	return TRUE;
+}
+
+static void set_powered(GDBusPendingPropertySet id, dbus_bool_t powered,
+								void *data)
+{
+	struct near_adapter *adapter = data;
+	int err;
+
+	err = __near_netlink_adapter_enable(adapter->idx, powered);
+	if (err < 0) {
+		if (err == -EALREADY) {
+			if (powered)
+				g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						"Device already enabled");
+			else
+				g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						"Device already disabled");
+		}
+
+		g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						strerror(err));
+
+		return;
+	}
+
+	g_dbus_pending_property_success(id);
+
+	adapter->powered = powered;
+
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Powered");
+}
+
+static void property_set_powered(const GDBusPropertyTable *property,
+					DBusMessageIter *value,
+					GDBusPendingPropertySet id, void *data)
+{
+	dbus_bool_t powered;
+
+	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN) {
+		g_dbus_pending_property_error(id,
+					NFC_ERROR_INTERFACE ".InvalidArguments",
+					"Invalid arguments in method call");
+		return;
+	}
+
+	dbus_message_iter_get_basic(value, &powered);
+
+	set_powered(id, powered, data);
+}
+
+static void append_protocols(DBusMessageIter *iter,
+					struct near_adapter *adapter)
+{
 	const char *str;
 
 	DBG("protocols 0x%x", adapter->protocols);
@@ -240,7 +308,8 @@ static void append_protocols(DBusMessageIter *iter, void *user_data)
 		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 
-	if (adapter->protocols & NFC_PROTO_ISO14443_MASK) {
+	if ((adapter->protocols & NFC_PROTO_ISO14443_MASK) ||
+	    (adapter->protocols & NFC_PROTO_ISO14443_B_MASK)) {
 		str = "ISO-DEP";
 
 		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
@@ -251,179 +320,28 @@ static void append_protocols(DBusMessageIter *iter, void *user_data)
 
 		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
-}
 
-static void append_tag_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_tag *tag = value;
-	DBusMessageIter *iter = user_data;
-	const char *tag_path;
+	if (adapter->protocols & NFC_PROTO_ISO15693_MASK) {
+		str = "ISO-15693";
 
-	tag_path = __near_tag_get_path(tag);
-	if (tag_path == NULL)
-		return;
-
-	DBG("%s", tag_path);
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH, &tag_path);
-}
-
-static void append_tags(DBusMessageIter *iter, void *user_data)
-{
-	struct near_adapter *adapter = user_data;
-
-	DBG("");
-
-	g_hash_table_foreach(adapter->tags, append_tag_path, iter);
-}
-
-static void append_device_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_device *device = value;
-	DBusMessageIter *iter = user_data;
-	const char *device_path;
-
-	device_path = __near_device_get_path(device);
-	if (device_path == NULL)
-		return;
-
-	DBG("%s", device_path);
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&device_path);
-}
-
-static void append_devices(DBusMessageIter *iter, void *user_data)
-{
-	struct near_adapter *adapter = user_data;
-
-	DBG("");
-
-	g_hash_table_foreach(adapter->devices, append_device_path, iter);
-}
-
-void __near_adapter_tags_changed(uint32_t adapter_idx)
-{
-	struct near_adapter *adapter;
-
-	DBG("");
-
-	adapter = g_hash_table_lookup(adapter_hash,
-					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
-		return;
-
-	near_dbus_property_changed_array(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Tags",
-					DBUS_TYPE_OBJECT_PATH, append_tags,
-					adapter);
-}
-
-void __near_adapter_devices_changed(uint32_t adapter_idx)
-{
-	struct near_adapter *adapter;
-
-	DBG("");
-
-	adapter = g_hash_table_lookup(adapter_hash,
-					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
-		return;
-
-	near_dbus_property_changed_array(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Devices",
-					DBUS_TYPE_OBJECT_PATH, append_devices,
-					adapter);
-}
-
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct near_adapter *adapter = data;
-	const char *rf_mode;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	near_dbus_dict_open(&array, &dict);
-
-	near_dbus_dict_append_basic(&dict, "Powered",
-				    DBUS_TYPE_BOOLEAN, &adapter->powered);
-
-	near_dbus_dict_append_basic(&dict, "Polling",
-				    DBUS_TYPE_BOOLEAN, &adapter->polling);
-
-	rf_mode = rf_mode_to_string(adapter);
-	if (rf_mode != NULL)
-		near_dbus_dict_append_basic(&dict, "Mode",
-						DBUS_TYPE_STRING, &rf_mode);
-
-	near_dbus_dict_append_array(&dict, "Protocols",
-				DBUS_TYPE_STRING, append_protocols, adapter);
-
-	near_dbus_dict_append_array(&dict, "Tags",
-				DBUS_TYPE_OBJECT_PATH, append_tags, adapter);
-
-	near_dbus_dict_append_array(&dict, "Devices",
-				DBUS_TYPE_OBJECT_PATH, append_devices, adapter);
-
-	near_dbus_dict_close(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct near_adapter *adapter = data;
-	DBusMessageIter iter, value;
-	const char *name;
-	int type, err;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __near_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	type = dbus_message_iter_get_arg_type(&value);
-
-	if (g_str_equal(name, "Powered") == TRUE) {
-		near_bool_t powered;
-
-		if (type != DBUS_TYPE_BOOLEAN)
-			return __near_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &powered);
-
-		err = __near_netlink_adapter_enable(adapter->idx, powered);
-		if (err < 0) {
-			if (err == -EALREADY) {
-				if (powered == TRUE)
-					return __near_error_already_enabled(msg);
-				else
-					return __near_error_already_disabled(msg);
-			}
-
-			return __near_error_failed(msg, -err);
-		}
-
-		adapter->powered = powered;
-	} else {
-		return __near_error_invalid_property(msg);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
+}
 
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+static gboolean property_get_protocols(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_adapter *adapter = user_data;
+	DBusMessageIter dict;
+
+	dbus_message_iter_open_container(iter, DBUS_TYPE_ARRAY,
+						DBUS_TYPE_STRING_AS_STRING, &dict);
+
+	append_protocols(&dict, adapter);
+
+	dbus_message_iter_close_container(iter, &dict);
+
+	return TRUE;
 }
 
 static DBusMessage *start_poll_loop(DBusConnection *conn,
@@ -435,6 +353,11 @@ static DBusMessage *start_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (!adapter->powered) {
+		near_error("Adapter is down, can not start polling");
+		return __near_error_failed(msg, ENODEV);
+	}
+
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &dbus_mode,
 							DBUS_TYPE_INVALID);
 
@@ -464,14 +387,14 @@ static DBusMessage *stop_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
-	if (adapter->polling == FALSE)
+	if (!adapter->polling)
 		return __near_error_not_polling(msg);
 
 	err = __near_netlink_stop_poll(adapter->idx);
 	if (err < 0)
 		return __near_error_failed(msg, -err);
 
-	adapter->polling = FALSE;
+	adapter->polling = false;
 
 	polling_changed(adapter);
 
@@ -489,11 +412,11 @@ static gboolean check_presence(gpointer user_data)
 
 	DBG("");
 
-	if (adapter == NULL)
+	if (!adapter)
 		return FALSE;
 
 	tag = adapter->tag_link;
-	if (tag == NULL)
+	if (!tag)
 		goto out_err;
 
 	err = __near_tag_check_presence(tag, tag_present_cb);
@@ -506,12 +429,26 @@ static gboolean check_presence(gpointer user_data)
 
 out_err:
 	near_adapter_disconnect(adapter->idx);
-	if (adapter->constant_poll == TRUE)
+	if (adapter->constant_poll)
 		adapter_start_poll(adapter);
 
 	return FALSE;
 }
 
+static gboolean dep_timer(gpointer user_data)
+{
+	struct near_adapter *adapter = user_data;
+
+	DBG("");
+
+	if (!adapter)
+		return FALSE;
+
+	adapter_start_poll(adapter);
+
+	return FALSE;
+}
+
 static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 								int status)
 {
@@ -521,14 +458,14 @@ static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
 		DBG("Tag is gone");
 
 		near_adapter_disconnect(adapter->idx);
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
@@ -548,7 +485,7 @@ void __near_adapter_start_check_presence(uint32_t adapter_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 			GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	adapter->presence_timeout =
@@ -565,7 +502,7 @@ void __near_adapter_stop_check_presence(uint32_t adapter_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 			GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (adapter->presence_timeout > 0)
@@ -573,46 +510,41 @@ void __near_adapter_stop_check_presence(uint32_t adapter_idx,
 }
 
 static const GDBusMethodTable adapter_methods[] = {
-	{ GDBUS_METHOD("GetProperties",
-				NULL, GDBUS_ARGS({"properties", "a{sv}"}),
-				get_properties) },
-	{ GDBUS_METHOD("SetProperty",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"}),
-				NULL, set_property) },
 	{ GDBUS_METHOD("StartPollLoop", GDBUS_ARGS({"name", "s"}), NULL,
 							start_poll_loop) },
 	{ GDBUS_METHOD("StopPollLoop", NULL, NULL, stop_poll_loop) },
 	{ },
 };
 
-static const GDBusSignalTable adapter_signals[] = {
-	{ GDBUS_SIGNAL("PropertyChanged",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"})) },
-	{ GDBUS_SIGNAL("TagFound", GDBUS_ARGS({"address", "o"})) },
-	{ GDBUS_SIGNAL("TagLost", GDBUS_ARGS({"address", "o"})) },
+static const GDBusPropertyTable adapter_properties[] = {
+	{ "Mode", "s", property_get_mode },
+	{ "Powered", "b", property_get_powered, property_set_powered },
+	{ "Polling", "b", property_get_polling },
+	{ "Protocols", "as", property_get_protocols },
+
 	{ }
 };
 
 struct near_adapter *__near_adapter_create(uint32_t idx,
-		const char *name, uint32_t protocols, near_bool_t powered)
+		const char *name, uint32_t protocols, bool powered)
 {
 	struct near_adapter *adapter;
-	near_bool_t powered_setting;
+	bool powered_setting;
 
 	adapter = g_try_malloc0(sizeof(struct near_adapter));
-	if (adapter == NULL)
+	if (!adapter)
 		return NULL;
 
 	adapter->name = g_strdup(name);
-	if (adapter->name == NULL) {
+	if (!adapter->name) {
 		g_free(adapter);
 		return NULL;
 	}
 
 	powered_setting = near_setting_get_bool("DefaultPowered");
-	if (powered_setting == TRUE && powered == FALSE &&
-	    !__near_netlink_adapter_enable(adapter->idx, powered_setting))
-			powered = TRUE;
+	if (powered_setting && !powered &&
+	    !__near_netlink_adapter_enable(idx, powered_setting))
+			powered = true;
 
 	DBG("Powered %d", powered);
 
@@ -620,7 +552,7 @@ struct near_adapter *__near_adapter_create(uint32_t idx,
 	adapter->protocols = protocols;
 	adapter->powered = powered;
 	adapter->constant_poll = near_setting_get_bool("ConstantPoll");
-	adapter->dep_up = FALSE;
+	adapter->dep_up = false;
 	adapter->tags = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 							NULL, free_tag);
 	adapter->tag_sock = -1;
@@ -651,42 +583,55 @@ struct near_adapter *__near_adapter_get(uint32_t idx)
 	return g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
 }
 
-int __near_adapter_set_dep_state(uint32_t idx, near_bool_t dep)
+int __near_adapter_set_dep_state(uint32_t idx, bool dep)
 {
 	struct near_adapter *adapter;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	adapter->dep_up = dep;
 
-	if (dep == FALSE && adapter->constant_poll == TRUE)
-		adapter_start_poll(adapter);
+	if (!dep && adapter->constant_poll) {
+		/*
+		 * The immediate polling may fail if the adapter is busy in
+		 * that very moment. In this case we need to try polling later
+		 * again, so constant polling will work properly.
+		 */
+		if(adapter_start_poll(adapter) == -EBUSY) {
+			near_error("Adapter is busy, retry polling later");
+			g_timeout_add_seconds(1, dep_timer, adapter);
+		}
+	}
 
-	if (dep == FALSE) {
+	if (!dep) {
 		uint32_t target_idx;
 
 		target_idx =  __neard_device_get_idx(adapter->device_link);
 		__near_adapter_remove_target(idx, target_idx);
 	} else {
-		__near_adapter_devices_changed(idx);
+		if (adapter->dep_timer > 0)
+			g_source_remove(adapter->dep_timer);
+
+		if (!__near_device_register_interface(adapter->device_link))
+			return -ENODEV;
 	}
 
 	return 0;
 }
 
-near_bool_t __near_adapter_get_dep_state(uint32_t idx)
+bool __near_adapter_get_dep_state(uint32_t idx)
 {
 	struct near_adapter *adapter;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
-		return FALSE;
+	if (!adapter)
+		return false;
 
 	return adapter->dep_up;
 }
@@ -697,7 +642,7 @@ int __near_adapter_add(struct near_adapter *adapter)
 
 	DBG("%s", adapter->path);
 
-	if (g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx)) != NULL)
+	if (g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx)))
 		return -EEXIST;
 
 	g_hash_table_insert(adapter_hash, GINT_TO_POINTER(idx), adapter);
@@ -706,8 +651,8 @@ int __near_adapter_add(struct near_adapter *adapter)
 
 	g_dbus_register_interface(connection, adapter->path,
 					NFC_ADAPTER_INTERFACE,
-					adapter_methods, adapter_signals,
-					NULL, adapter, NULL);
+					adapter_methods, NULL,
+					adapter_properties, adapter, NULL);
 
 	return 0;
 }
@@ -730,19 +675,17 @@ static void tag_read_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
 		near_adapter_disconnect(adapter->idx);
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
 	}
 
-	__near_adapter_tags_changed(adapter_idx);
-
 	adapter->presence_timeout =
 		g_timeout_add_seconds(CHECK_PRESENCE_PERIOD,
 					check_presence, adapter);
@@ -757,16 +700,16 @@ static void device_read_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
-		if (adapter->device_link != NULL) {
+		if (adapter->device_link) {
 			__near_netlink_dep_link_down(adapter->idx);
 			adapter->device_link = NULL;
 		}
 
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
@@ -776,7 +719,9 @@ static void device_read_cb(uint32_t adapter_idx, uint32_t target_idx,
 static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 			uint32_t protocols,
 			uint16_t sens_res, uint8_t sel_res,
-			uint8_t *nfcid, uint8_t nfcid_len)
+			uint8_t *nfcid, uint8_t nfcid_len,
+			uint8_t iso15693_dsfid,
+			uint8_t iso15693_uid_len, uint8_t *iso15693_uid)
 {
 	struct near_tag *tag;
 	uint32_t tag_type;
@@ -784,8 +729,9 @@ static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 
 	tag = __near_tag_add(adapter->idx, target_idx, protocols,
 				sens_res, sel_res,
-				nfcid, nfcid_len);
-	if (tag == NULL)
+				nfcid, nfcid_len,
+				iso15693_dsfid, iso15693_uid_len, iso15693_uid);
+	if (!tag)
 		return -ENODEV;
 
 	g_hash_table_insert(adapter->tags, GINT_TO_POINTER(target_idx), tag);
@@ -816,8 +762,10 @@ static int adapter_add_device(struct near_adapter *adapter,
 	struct near_device *device;
 	int err;
 
+	DBG();
+
 	device = __near_device_add(adapter->idx, target_idx, nfcid, nfcid_len);
-	if (device == NULL)
+	if (!device)
 		return -ENODEV;
 
 	g_hash_table_insert(adapter->devices, GINT_TO_POINTER(target_idx),
@@ -832,8 +780,12 @@ static int adapter_add_device(struct near_adapter *adapter,
 
 	adapter->device_link = device;
 
-	if (adapter->dep_up == TRUE)
+	if (adapter->dep_up) {
+		if (!__near_device_register_interface(device))
+			return -ENODEV;
+
 		return 0;
+	}
 
 	err = __near_netlink_dep_link_up(adapter->idx, target_idx,
 					NFC_COMM_ACTIVE, NFC_RF_INITIATOR);
@@ -841,33 +793,47 @@ static int adapter_add_device(struct near_adapter *adapter,
 	if (err < 0)
 		adapter->device_link = NULL;
 
+	DBG("Starting DEP timer");
+
+	adapter->dep_timer = g_timeout_add_seconds(1, dep_timer, adapter);
+
 	return err;
 }
 
 int __near_adapter_add_target(uint32_t idx, uint32_t target_idx,
 			uint32_t protocols, uint16_t sens_res, uint8_t sel_res,
-			uint8_t *nfcid, uint8_t nfcid_len)
+			uint8_t *nfcid, uint8_t nfcid_len,
+			uint8_t iso15693_dsfid,
+			uint8_t iso15693_uid_len, uint8_t *iso15693_uid)
 {
 	struct near_adapter *adapter;
+	int ret;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	adapter->polling = FALSE;
+	adapter->polling = false;
 	polling_changed(adapter);
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_INITIATOR;
 	rf_mode_changed(adapter);
 
 	if (protocols & NFC_PROTO_NFC_DEP_MASK)
-		return adapter_add_device(adapter, target_idx,
+		ret = adapter_add_device(adapter, target_idx,
 						nfcid, nfcid_len);
 	else
-		return adapter_add_tag(adapter, target_idx, protocols,
-					sens_res, sel_res, nfcid, nfcid_len);
+		ret = adapter_add_tag(adapter, target_idx, protocols,
+					sens_res, sel_res, nfcid, nfcid_len,
+					iso15693_dsfid,
+					iso15693_uid_len, iso15693_uid);
+
+	if (ret < 0 && adapter->constant_poll)
+		adapter_start_poll(adapter);
+
+	return ret;
 }
 
 int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
@@ -877,25 +843,62 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
 
-	if (g_hash_table_remove(adapter->tags,
-			GINT_TO_POINTER(target_idx)) == TRUE) {
-		__near_adapter_tags_changed(idx);
-
+	if (g_hash_table_remove(adapter->tags, GINT_TO_POINTER(target_idx)))
 		return 0;
+
+	if (g_hash_table_remove(adapter->devices, GINT_TO_POINTER(target_idx)))
+		return 0;
+
+	return 0;
+}
+
+static gboolean poll_error(gpointer user_data)
+{
+	struct near_adapter *adapter = user_data;
+	bool reset;
+
+	DBG("adapter %d", adapter->idx);
+
+	reset = near_setting_get_bool("ResetOnError");
+	if (reset) {
+		near_error("Resetting nfc%d", adapter->idx);
+		 __near_netlink_adapter_enable(adapter->idx, false);
+		 __near_netlink_adapter_enable(adapter->idx, true);
 	}
 
-	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(target_idx)) == TRUE) {
-		__near_adapter_devices_changed(idx);
+	adapter_start_poll(adapter);
 
+	return FALSE;
+}
+
+int __near_adapter_get_targets_done(uint32_t idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("idx %d", idx);
+
+	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
+	if (!adapter)
+		return -ENODEV;
+
+	if (g_hash_table_size(adapter->devices) > 0)
 		return 0;
-	}
+
+	if (g_hash_table_size(adapter->tags) > 0)
+		return 0;
+
+	near_error("No targets found - Polling error");
+
+	adapter->polling = false;
+	polling_changed(adapter);
+
+	g_idle_add(poll_error, adapter);
 
 	return 0;
 }
@@ -908,11 +911,11 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	adapter->polling = FALSE;
-	adapter->dep_up = TRUE;
+	adapter->polling = false;
+	adapter->dep_up = true;
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_TARGET;
 	polling_changed(adapter);
 	rf_mode_changed(adapter);
@@ -921,8 +924,6 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 	if (ret < 0)
 		return ret;
 
-	__near_adapter_devices_changed(idx);
-
 	return 0;
 }
 
@@ -934,20 +935,18 @@ int __near_adapter_remove_device(uint32_t idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(device_idx)) == FALSE)
+	if (!g_hash_table_remove(adapter->devices, GINT_TO_POINTER(device_idx)))
 		return 0;
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
-	__near_adapter_devices_changed(idx);
 
-	adapter->dep_up = FALSE;
+	adapter->dep_up = false;
 
-	if (adapter->constant_poll == TRUE)
+	if (adapter->constant_poll)
 		adapter_start_poll(adapter);
 
 	return 0;
@@ -960,7 +959,7 @@ static void adapter_flush_rx(struct near_adapter *adapter, int error)
 	for (list = adapter->ioreq_list; list; list = list->next) {
 		struct near_adapter_ioreq *req = list->data;
 
-		if (req == NULL)
+		if (!req)
 			continue;
 
 		req->cb(NULL, error, req->data);
@@ -1009,7 +1008,7 @@ static gboolean adapter_recv_event(GIOChannel *channel, GIOCondition condition,
 
 	sk = g_io_channel_unix_get_fd(channel);
 	first = g_list_first(adapter->ioreq_list);
-	if (first == NULL)
+	if (!first)
 		return TRUE;
 
 	req = first->data;
@@ -1032,7 +1031,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	if (adapter->tag_sock != -1)
@@ -1040,7 +1039,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 
 	tag = g_hash_table_lookup(adapter->tags,
 				GINT_TO_POINTER(target_idx));
-	if (tag == NULL)
+	if (!tag)
 		return -ENOLINK;
 
 	sock = socket(AF_NFC, SOCK_SEQPACKET, NFC_SOCKPROTO_RAW);
@@ -1061,7 +1060,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 	adapter->tag_sock = sock;
 	adapter->tag_link = tag;
 
-	if (adapter->channel == NULL)
+	if (!adapter->channel)
 		adapter->channel = g_io_channel_unix_new(adapter->tag_sock);
 
 	g_io_channel_set_flags(adapter->channel, G_IO_FLAG_NONBLOCK, NULL);
@@ -1084,12 +1083,12 @@ int near_adapter_disconnect(uint32_t idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	DBG("link %p", adapter->tag_link);
 
-	if (adapter->tag_link == NULL)
+	if (!adapter->tag_link)
 		return -ENOLINK;
 
 	tag_type = __near_tag_get_type(adapter->tag_link);
@@ -1125,19 +1124,19 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL) {
+	if (!adapter) {
 		err = -ENODEV;
 		goto out_err;
 	}
 
-	if (adapter->tag_sock == -1 || adapter->tag_link == NULL) {
+	if (adapter->tag_sock == -1 || !adapter->tag_link) {
 		err = -ENOLINK;
 		goto out_err;
 	}
 
-	if (cb != NULL && adapter->watch != 0) {
+	if (cb && adapter->watch != 0) {
 		req = g_try_malloc0(sizeof(*req));
-		if (req == NULL) {
+		if (!req) {
 			err = -ENOMEM;
 			goto out_err;
 		}
@@ -1159,7 +1158,7 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	return err;
 
 out_err:
-	if (req != NULL) {
+	if (req) {
 		GList *last = g_list_last(adapter->ioreq_list);
 
 		g_free(req);
@@ -1167,7 +1166,7 @@ out_err:
 				g_list_delete_link(adapter->ioreq_list, last);
 	}
 
-	if (data_rel != NULL)
+	if (data_rel)
 		return (*data_rel)(err, data);
 
 	return err;
@@ -1180,7 +1179,7 @@ static void adapter_listen(gpointer key, gpointer value, gpointer user_data)
 
 	DBG("%s", adapter->path);
 
-	if (adapter->path == NULL)
+	if (!adapter->path)
 		return;
 
 	driver->listen(adapter->idx, device_read_cb);
