@@ -2,7 +2,7 @@
  *
  *  neard - Near Field Communication manager
  *
- *  Copyright (C) 2011  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2011-2016  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -149,6 +149,8 @@ struct near_ndef_mime_payload {
 		enum handover_carrier carrier_type;
 		uint16_t properties;	/* e.g.: NO_PAIRING_KEY */
 	} handover;
+	uint8_t *payload;
+	uint32_t payload_len;
 };
 
 /* Handover record definitions */
@@ -375,7 +377,7 @@ static gboolean property_get_type(const GDBusPropertyTable *property,
 		type = "AAR";
 		break;
 	}
-	
+
 	if (!type)
 		return FALSE;
 
@@ -637,6 +639,35 @@ static gboolean property_get_aar(const GDBusPropertyTable *property,
 	return TRUE;
 }
 
+static gboolean mime_payload_exists(const GDBusPropertyTable *property,
+					void *data)
+{
+	struct near_ndef_record *record = data;
+	gboolean exists = record->mime && record->mime->payload_len
+						&& record->mime->payload;
+
+	DBG("%s", exists ? "" : "No MIME payload");
+
+	return exists;
+}
+
+static gboolean property_get_mime_payload(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_ndef_record *record = user_data;
+	DBusMessageIter array;
+
+	DBG("");
+
+	dbus_message_iter_open_container(iter, DBUS_TYPE_ARRAY,
+			DBUS_TYPE_BYTE_AS_STRING, &array);
+	dbus_message_iter_append_fixed_array(&array, DBUS_TYPE_BYTE,
+			&record->mime->payload, record->mime->payload_len);
+	dbus_message_iter_close_container(iter, &array);
+
+	return TRUE;
+}
+
 static const GDBusPropertyTable record_properties[] = {
 
 	{ "Type", "s", property_get_type },
@@ -649,6 +680,8 @@ static const GDBusPropertyTable record_properties[] = {
 	{ "Size", "u", property_get_size, NULL, sp_size_exists },
 	{ "MIME", "s", property_get_mime, NULL, mime_exists },
 	{ "AAR", "s", property_get_aar, NULL, aar_exists },
+	{ "MIMEPayload", "ay", property_get_mime_payload, NULL,
+						mime_payload_exists },
 	{ }
 };
 
@@ -697,6 +730,7 @@ static void free_mime_payload(struct near_ndef_mime_payload *mime)
 	if (!mime)
 		return;
 
+	g_free(mime->payload);
 	g_free(mime->type);
 	g_free(mime);
 }
@@ -1518,10 +1552,20 @@ static struct near_ndef_mime_payload *parse_mime_type(
 	} else {
 		g_free(c_temp);
 		c_temp = NULL;
-		*c_data = NULL;
-		return mime;
-	}
 
+		if (!record->header->payload_len)
+			goto fail;
+
+		mime->payload = g_try_malloc0(record->header->payload_len);
+		if (!mime->payload) {
+			near_error("Memory allocation failed (MIME payload)");
+			goto fail;
+		}
+
+		mime->payload_len = record->header->payload_len;
+		memcpy(mime->payload, ndef_data + offset, mime->payload_len);
+	}
+fail:
 	*c_data = c_temp;
 
 	return mime;
@@ -2974,7 +3018,7 @@ int near_ndef_count_records(uint8_t *ndef_in, size_t ndef_in_length,
 	int err;
 	size_t offset;
 	struct near_ndef_record *record = NULL;
-	int counted_records = 0 ;
+	int counted_records = 0;
 
 	DBG("");
 
@@ -3159,9 +3203,9 @@ fail:
 	return NULL;
 }
 
-static char *get_text_field(DBusMessage *msg, char *text)
+static char *get_text_field(DBusMessageIter *msg, char *text)
 {
-	DBusMessageIter iter, arr_iter;
+	DBusMessageIter arr_iter;
 	char *uri = NULL;
 
 	DBG("");
@@ -3169,8 +3213,7 @@ static char *get_text_field(DBusMessage *msg, char *text)
 	if (!text)
 		return NULL;
 
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &arr_iter);
+	dbus_message_iter_recurse(msg, &arr_iter);
 
 	while (dbus_message_iter_get_arg_type(&arr_iter) !=
 					DBUS_TYPE_INVALID) {
@@ -3197,12 +3240,12 @@ static char *get_text_field(DBusMessage *msg, char *text)
 	return uri;
 }
 
-static inline char *get_uri_field(DBusMessage *msg)
+static inline char *get_uri_field(DBusMessageIter *msg)
 {
 	return get_text_field(msg, "URI");
 }
 
-static GSList *get_carrier_field(DBusMessage *msg)
+static GSList *get_carrier_field(DBusMessageIter *msg)
 {
 	char *carrier;
 	char **arr;
@@ -3226,15 +3269,14 @@ static GSList *get_carrier_field(DBusMessage *msg)
 	return carriers;
 }
 
-static struct near_ndef_message *build_text_record(DBusMessage *msg)
+static struct near_ndef_message *build_text_record(DBusMessageIter *msg)
 {
-	DBusMessageIter iter, arr_iter;
+	DBusMessageIter arr_iter;
 	char *cod = NULL, *lang = NULL, *rep = NULL;
 
 	DBG("");
 
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &arr_iter);
+	dbus_message_iter_recurse(msg, &arr_iter);
 
 	while (dbus_message_iter_get_arg_type(&arr_iter) !=
 					DBUS_TYPE_INVALID) {
@@ -3265,7 +3307,7 @@ static struct near_ndef_message *build_text_record(DBusMessage *msg)
 	return near_ndef_prepare_text_record(cod, lang, rep);
 }
 
-static struct near_ndef_message *build_uri_record(DBusMessage *msg)
+static struct near_ndef_message *build_uri_record(DBusMessageIter *msg)
 {
 	char *uri = NULL;
 	const char *uri_prefix = NULL;
@@ -3299,7 +3341,7 @@ static struct near_ndef_message *build_uri_record(DBusMessage *msg)
 						(uint8_t *)(uri + id_len));
 }
 
-static struct near_ndef_message *build_sp_record(DBusMessage *msg)
+static struct near_ndef_message *build_sp_record(DBusMessageIter *msg)
 {
 	char *uri = NULL;
 	const char *uri_prefix;
@@ -3336,7 +3378,7 @@ static struct near_ndef_message *build_sp_record(DBusMessage *msg)
 }
 
 static struct near_ndef_message *build_ho_record(enum record_type type,
-							DBusMessage *msg)
+							DBusMessageIter *msg)
 {
 	struct near_ndef_message *ho;
 	GSList *carriers;
@@ -3497,7 +3539,8 @@ static char *get_mime_payload_data(DBusMessageIter iter,
 		dbus_message_iter_next(&ent_iter);
 		dbus_message_iter_recurse(&ent_iter, &var_iter);
 
-		if (g_strcmp0(key, "Payload") == 0) {
+		if (g_strcmp0(key, "Payload") == 0 ||
+				g_strcmp0(key, "MIMEPayload") == 0) {
 			if (dbus_message_iter_get_arg_type(&var_iter) ==
 							DBUS_TYPE_ARRAY &&
 			    dbus_message_iter_get_element_type(&var_iter) ==
@@ -3536,15 +3579,14 @@ static struct near_ndef_message *near_ndef_prepare_mime_payload_record(
 	return mime;
 }
 
-static struct near_ndef_message *build_mime_record(DBusMessage *msg)
+static struct near_ndef_message *build_mime_record(DBusMessageIter *msg)
 {
 	DBusMessageIter iter, arr_iter;
 	char *key, *mime_str, *ssid, *passphrase;
 
 	DBG("");
 
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &arr_iter);
+	dbus_message_iter_recurse(msg, &arr_iter);
 
 	while (dbus_message_iter_get_arg_type(&arr_iter) !=
 					DBUS_TYPE_INVALID) {
@@ -3618,18 +3660,97 @@ static struct near_ndef_message *build_mime_record(DBusMessage *msg)
 	return NULL;
 }
 
-struct near_ndef_message *__ndef_build_from_message(DBusMessage *msg)
+static gboolean near_dbus_message_iter_recurse_to_signature(
+					DBusMessageIter *iter, char *sig)
+{
+	gboolean found = FALSE;
+	int type;
+
+	while ((type = dbus_message_iter_get_arg_type(iter))
+			!= DBUS_TYPE_INVALID) {
+
+		if (g_strcmp0(sig, dbus_message_iter_get_signature(iter))
+				== 0) {
+			found = TRUE;
+			break;
+		}
+
+		if (dbus_type_is_container(type))
+			dbus_message_iter_recurse(iter, iter);
+		else if (dbus_message_iter_has_next(iter))
+			dbus_message_iter_next(iter);
+		else break;
+	}
+
+	DBG("sig %s %s", sig, found ? "found" : "not found");
+
+	return found;
+}
+
+static gboolean near_dbus_message_with_multiple_records(DBusMessage *msg)
 {
 	DBusMessageIter iter;
-	DBusMessageIter arr_iter;
-	struct near_ndef_message *ndef;
+	char *s;
+
+	dbus_message_iter_init(msg, &iter);
+
+	near_dbus_message_iter_recurse_to_signature(&iter, "s");
+
+	dbus_message_iter_get_basic(&iter, &s);
+
+	return g_strcmp0(s, "Records") == 0;
+}
+
+static gboolean near_dbus_message_get_records(DBusMessage *msg,
+						DBusMessageIter *iter)
+{
+	gboolean found = FALSE;
+
+	dbus_message_iter_init(msg, iter);
+again:
+	if (near_dbus_message_iter_recurse_to_signature(iter, "a{sv}")) {
+
+		if (msg && near_dbus_message_with_multiple_records(msg)) {
+
+			near_dbus_message_iter_recurse_to_signature(iter, "v");
+
+			msg = NULL;
+
+			goto again;
+		}
+
+		found = TRUE;
+	}
+
+	return found;
+}
+
+static struct near_ndef_message *build_multi_record_message(
+				DBusMessageIter *iter, const char *type)
+{
+	struct near_ndef_message *ndef = NULL;
 
 	DBG("");
 
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &arr_iter);
+	if (g_strcmp0(type, "SmartPoster") == 0)
+		ndef = build_sp_record(iter);
+	else if (g_strcmp0(type, "Handover") == 0)
+		ndef = build_ho_record(RECORD_TYPE_WKT_HANDOVER_REQUEST, iter);
+	else if (g_strcmp0(type, "StaticHandover") == 0)
+		ndef = build_ho_record(RECORD_TYPE_WKT_HANDOVER_SELECT, iter);
 
-	ndef = NULL;
+	return ndef;
+}
+
+static struct near_ndef_message *ndef_build_from_record(DBusMessageIter *msg,
+							bool begin, bool end)
+{
+	struct near_ndef_message *ndef = NULL;
+	DBusMessageIter arr_iter;
+
+	DBG("");
+
+	dbus_message_iter_recurse(msg, &arr_iter);
 
 	while (dbus_message_iter_get_arg_type(&arr_iter) !=
 						DBUS_TYPE_INVALID) {
@@ -3658,20 +3779,21 @@ struct near_ndef_message *__ndef_build_from_message(DBusMessage *msg)
 			} else if (g_strcmp0(value, "URI") == 0) {
 				ndef = build_uri_record(msg);
 				break;
-			} else if (g_strcmp0(value, "SmartPoster") == 0) {
-				ndef = build_sp_record(msg);
-				break;
-			} else if (g_strcmp0(value, "Handover") == 0) {
-				ndef = build_ho_record(
-					RECORD_TYPE_WKT_HANDOVER_REQUEST, msg);
-				break;
-			} else if (g_strcmp0(value, "StaticHandover") == 0) {
-				ndef = build_ho_record(
-					RECORD_TYPE_WKT_HANDOVER_SELECT, msg);
-				break;
 			} else if (g_strcmp0(value, "MIME") == 0) {
 				ndef = build_mime_record(msg);
 				break;
+			} else if ((begin && end) &&
+					(g_strcmp0(value, "SmartPoster") == 0 ||
+					g_strcmp0(value, "Handover") == 0 ||
+					g_strcmp0(value, "StaticHandover")
+						== 0)) {
+				/*
+				 * Currently SmartPoster and Handover messages
+				 * are supported only in the single record mode
+				 * of the tag.write() method.
+				 */
+				ndef = build_multi_record_message(msg, value);
+				break;
 			} else {
 				near_error("%s not supported", value);
 				ndef = NULL;
@@ -3687,6 +3809,77 @@ struct near_ndef_message *__ndef_build_from_message(DBusMessage *msg)
 	return ndef;
 }
 
+static gboolean ndef_message_append(struct near_ndef_message *to,
+					struct near_ndef_message *from)
+{
+	gboolean ret = TRUE;
+	uint8_t *data = g_try_realloc(to->data, to->length + from->length);
+
+	if (!data) {
+		near_error("Memory re-allocation failed (NDEF message data)");
+		ret = FALSE;
+		goto exit;
+	}
+
+	to->data = data;
+
+	memcpy(to->data + to->length, from->data, from->length);
+
+	to->offset = to->length + from->offset;
+
+	to->length += from->length;
+exit:
+	return ret;
+}
+
+struct near_ndef_message *__ndef_build_from_message(DBusMessage *msg)
+{
+	struct near_ndef_message *ndef, *record;
+	DBusMessageIter iter;
+	bool begin = TRUE, end = FALSE;
+
+	DBG("");
+
+	if (!near_dbus_message_get_records(msg, &iter)) {
+		near_error("No records found");
+		ndef = NULL;
+		goto exit;
+	}
+
+	for (ndef = NULL ;; dbus_message_iter_next(&iter)) {
+
+		end = !dbus_message_iter_has_next(&iter);
+
+		record = ndef_build_from_record(&iter, begin, end);
+		if (!record)
+			goto err;
+
+		near_ndef_set_mb_me(record->data, begin, end);
+
+		if (!ndef) {
+			ndef = record;
+			goto next;
+		}
+
+		if (!ndef_message_append(ndef, record))
+			goto err;
+
+		free_ndef_message(record);
+	next:
+		if (end)
+			break;
+
+		begin = FALSE;
+	}
+exit:
+	return ndef;
+err:
+	free_ndef_message(ndef);
+	free_ndef_message(record);
+	ndef = NULL;
+	goto exit;
+}
+
 int __near_ndef_init(void)
 {
 	DBG("");
