@@ -68,6 +68,12 @@ struct nlnfc_state {
 static struct nlnfc_state *nfc_state;
 static GIOChannel *netlink_channel = NULL;
 
+struct send_msg_data {
+	void *data;
+	int *done;
+	int (*finish_handler)(struct nl_msg *, void *);
+};
+
 static int error_handler(struct sockaddr_nl *nla, struct nlmsgerr *err,
 			 void *arg)
 {
@@ -80,13 +86,17 @@ static int error_handler(struct sockaddr_nl *nla, struct nlmsgerr *err,
 	return NL_STOP;
 }
 
-static int finish_handler(struct nl_msg *msg, void *arg)
+static int __finish_handler(struct nl_msg *msg, void *arg)
 {
-	int *ret = arg;
+	struct send_msg_data *data = arg;
+
 
 	DBG("");
 
-	*ret = 1;
+	if (data->finish_handler)
+		data->finish_handler(msg, data->data);
+
+	*(data->done) = 1;
 
 	return NL_SKIP;
 }
@@ -102,12 +112,14 @@ static int ack_handler(struct nl_msg *msg, void *arg)
 	return NL_STOP;
 }
 
-static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
+static int __nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 			int (*rx_handler)(struct nl_msg *, void *),
+			int (*finish_handler)(struct nl_msg *, void *),
 			void *data)
 {
 	struct nl_cb *cb;
 	int err, done;
+	struct send_msg_data send_data;
 
 	DBG("");
 
@@ -124,9 +136,12 @@ static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 	}
 
 	err = done = 0;
+	send_data.done = &done;
+	send_data.data = data;
+	send_data.finish_handler = finish_handler;
 
+	nl_cb_set(cb, NL_CB_FINISH, NL_CB_CUSTOM, __finish_handler, &send_data);
 	nl_cb_err(cb, NL_CB_CUSTOM, error_handler, &err);
-	nl_cb_set(cb, NL_CB_FINISH, NL_CB_CUSTOM, finish_handler, &done);
 	nl_cb_set(cb, NL_CB_ACK, NL_CB_CUSTOM, ack_handler, &done);
 
 	if (rx_handler)
@@ -140,6 +155,13 @@ static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 	return err;
 }
 
+static inline int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
+			int (*rx_handler)(struct nl_msg *, void *),
+			void *data)
+{
+	return __nl_send_msg(sock, msg, rx_handler, NULL, data);
+}
+
 static int get_devices_handler(struct nl_msg *n, void *arg)
 {
 	struct nlmsghdr *nlh = nlmsg_hdr(n);
@@ -272,6 +294,41 @@ nla_put_failure:
 	return err;
 }
 
+int __near_netlink_activate_target(uint32_t idx, uint32_t target_idx,
+				   uint32_t protocol)
+{
+	struct nl_msg *msg;
+	void *hdr;
+	int err;
+
+	DBG("");
+
+	msg = nlmsg_alloc();
+	if (!msg)
+		return -ENOMEM;
+
+	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
+			NLM_F_REQUEST, NFC_CMD_ACTIVATE_TARGET,
+			NFC_GENL_VERSION);
+	if (!hdr) {
+		err = -EINVAL;
+		goto nla_put_failure;
+	}
+
+	err = -EMSGSIZE;
+
+	NLA_PUT_U32(msg, NFC_ATTR_DEVICE_INDEX, idx);
+	NLA_PUT_U32(msg, NFC_ATTR_TARGET_INDEX, target_idx);
+	NLA_PUT_U32(msg, NFC_ATTR_PROTOCOLS, protocol);
+
+	err = nl_send_msg(nfc_state->cmd_sock, msg, NULL, NULL);
+
+nla_put_failure:
+	nlmsg_free(msg);
+
+	return err;
+}
+
 int __near_netlink_dep_link_up(uint32_t idx, uint32_t target_idx,
 				uint8_t comm_mode, uint8_t rf_mode)
 {
@@ -435,6 +492,8 @@ static int get_targets_handler(struct nl_msg *n, void *arg)
 	uint16_t sens_res = 0;
 	uint8_t sel_res = 0;
 	uint8_t nfcid[NFC_MAX_NFCID1_LEN], nfcid_len;
+	uint8_t iso15693_dsfid = 0;
+	uint8_t iso15693_uid_len, iso15693_uid[NFC_MAX_ISO15693_UID_LEN];
 
 	DBG("");
 
@@ -461,15 +520,43 @@ static int get_targets_handler(struct nl_msg *n, void *arg)
 		nfcid_len = 0;
 	}
 
+	if (attrs[NFC_ATTR_TARGET_ISO15693_DSFID])
+		iso15693_dsfid =
+			nla_get_u8(attrs[NFC_ATTR_TARGET_ISO15693_DSFID]);
+
+	if (attrs[NFC_ATTR_TARGET_ISO15693_UID]) {
+		iso15693_uid_len = nla_len(attrs[NFC_ATTR_TARGET_ISO15693_UID]);
+		if (iso15693_uid_len == NFC_MAX_ISO15693_UID_LEN)
+			memcpy(iso15693_uid,
+			       nla_data(attrs[NFC_ATTR_TARGET_ISO15693_UID]),
+					NFC_MAX_ISO15693_UID_LEN);
+	} else {
+		iso15693_uid_len = 0;
+	}
+
 	DBG("target idx %d proto 0x%x sens_res 0x%x sel_res 0x%x NFCID len %d",
 	    target_idx, protocols, sens_res, sel_res, nfcid_len);
+	DBG("\tiso15693_uid_len %d", iso15693_uid_len);
 
 	__near_adapter_add_target(adapter_idx, target_idx, protocols,
-				  sens_res, sel_res, nfcid, nfcid_len);
+				  sens_res, sel_res, nfcid, nfcid_len,
+				  iso15693_dsfid,
+				  iso15693_uid_len, iso15693_uid);
 
 	return 0;
 }
 
+static int get_targets_finish_handler(struct nl_msg *n, void *arg)
+{
+	uint32_t adapter_idx;
+
+	DBG("");
+
+	adapter_idx = *((uint32_t *)arg);
+
+	return __near_adapter_get_targets_done(adapter_idx);
+}
+
 static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 {
 	struct nlattr *attr[NFC_ATTR_MAX + 1];
@@ -504,8 +591,9 @@ static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 
 	NLA_PUT_U32(msg, NFC_ATTR_DEVICE_INDEX, adapter_idx);
 
-	err = nl_send_msg(nfc_state->cmd_sock, msg,
-				get_targets_handler, &adapter_idx);
+	err = __nl_send_msg(nfc_state->cmd_sock, msg,
+				get_targets_handler, get_targets_finish_handler,
+								&adapter_idx);
 
 nla_put_failure:
 	nlmsg_free(msg);
@@ -642,10 +730,17 @@ static int nfc_netlink_event_tm_deactivated(struct genlmsghdr *gnlh)
 
 static int nfc_netlink_event(struct nl_msg *n, void *arg)
 {
+	struct sockaddr_nl *src = nlmsg_get_src(n);
 	struct genlmsghdr *gnlh = nlmsg_data(nlmsg_hdr(n));
 
 	DBG("event 0x%x", gnlh->cmd);
 
+	if (src->nl_pid) {
+		near_error("WARNING: Wrong netlink message sender %d",
+								src->nl_pid);
+		return NL_SKIP;
+	}
+
 	switch (gnlh->cmd) {
 	case NFC_EVENT_TARGETS_FOUND:
 		DBG("Targets found");
